body {
    overflow-x: hidden;
}

#loader {
    position: fixed;
    top: 0;
    left: 0;
    height: 100%;
    width: 100%;
    background-color: rgba(0,0,0,0.1);
    z-index: 1000000;
}

#loader .spinner {
    width: 100%;
    height: 100%;
    background: url('/static/vendor/loader.gif') no-repeat center;
    background-size: 10%;
    transition: opacity 1s linear;
}

.nav, .pagination, .carousel, .panel-title a {
    cursor: pointer;
}

#index-page .containers .container-item {
    margin: 3px;
    padding: 5px;
    cursor: pointer;
}

#index-page .containers .container-name {
}

#index-page .containers .container-name i {
    padding: 0 5px 0 0;
}

#index-page .containers .container-active {
    border-radius: 3px;
    background-color: #2e6da4;
    color: #ffffff;
}

input.ng-invalid {
    box-shadow: 0px 0px 1px 1px #ff0000;
}
